@@ -1,10 +1,11 @@
 ;;; mastodon-media.el --- Functions for inlining Mastodon media  -*- lexical-binding: t -*-
 
 ;; Copyright (C) 2017-2019 Johnson Denen
+;; Copyright (C) 2020-2022 Marty Hiatt
 ;; Author: Johnson Denen <johnson.denen@gmail.com>
+;;         Marty Hiatt <martianhiatus@riseup.net>
 ;; Maintainer: Marty Hiatt <martianhiatus@riseup.net>
-;; Version: 0.10.0
-;; Package-Requires: ((emacs "27.1"))
+;; Version: 1.0.0
 ;; Homepage: https://codeberg.org/martianh/mastodon.el
 
 ;; This file is not part of GNU Emacs.
@@ -35,6 +36,8 @@
 ;;; Code:
 (require 'url-cache)
 
+(autoload 'mastodon-tl--propertize-img-str-or-url "mastodon-tl")
+
 (defvar url-show-status)
 
 (defvar mastodon-tl--shr-image-map-replacement)
@@ -46,17 +49,14 @@
 
 (defcustom mastodon-media--avatar-height 20
   "Height of the user avatar images (if shown)."
-  :group 'mastodon-media
   :type 'integer)
 
 (defcustom mastodon-media--preview-max-height 250
   "Max height of any media attachment preview to be shown in timelines."
-  :group 'mastodon-media
   :type 'integer)
 
 (defcustom mastodon-media--enable-image-caching nil
   "Whether images should be cached."
-  :group 'mastodon-media
   :type 'boolean)
 
 (defvar mastodon-media--generic-avatar-data
@@ -135,58 +135,54 @@ CAQgEIBAAAIBFiNOFMaY6V1tnFhkDQIQCEAgAIEABAKAQAACAQgEIBCAQAACAQgEIBCAQABIXO4e
 c1y+zhoEIBCAQAAQCEAgAIEABAIQCEAgAIEABAIQCEAgAAgEIBCAQAACAQgEIBCAQAACAQgEAIEA
 BAIQCEAgAIEABAIsJVH58WqHw8FIgjUIQCAACAQgEIBAAAIBCAQgEIBAAAIBCAQgEAAEAhAIQCBA
 fKRJkmVZjAQwh78A6vCRWJE8K+8AAAAASUVORK5CYII=")
-  "The PNG data for a generic 200x200 'broken image' view.")
+  "The PNG data for a generic 200x200 \"broken image\" view.")
 
 (defun mastodon-media--process-image-response
     (status-plist marker image-options region-length url)
   "Callback function processing the url retrieve response for URL.
-
 STATUS-PLIST is the usual plist of status events as per `url-retrieve'.
 IMAGE-OPTIONS are the precomputed options to apply to the image.
 MARKER is the marker to where the response should be visible.
 REGION-LENGTH is the length of the region that should be replaced
 with the image."
-  (when (marker-buffer marker) ; only if the buffer hasn't been kill in the meantime
+  (when (marker-buffer marker) ; if buffer hasn't been killed
     (let ((url-buffer (current-buffer))
           (is-error-response-p (eq :error (car status-plist))))
-      (unwind-protect
-          (let* ((data (unless is-error-response-p
-                         (goto-char (point-min))
-                         (search-forward "\n\n")
-                         (buffer-substring (point) (point-max))))
-                 (image (when data
-                          (apply #'create-image data
-                                 (if (version< emacs-version "27.1")
-                                     (when image-options 'imagemagick)
-                                   nil) ; inbuilt scaling in 27.1
-                                 t image-options))))
-            (when mastodon-media--enable-image-caching
-              (unless (url-is-cached url) ; cache if not already cached
-                (url-store-in-cache url-buffer)))
-            (with-current-buffer (marker-buffer marker)
-              ;; Save narrowing in our buffer
-              (let ((inhibit-read-only t))
-                (save-restriction
-                  (widen)
-                  (put-text-property marker
-                                     (+ marker region-length) 'media-state 'loaded)
-                  (when image
-                    ;; We only set the image to display if we could load
-                    ;; it; we already have set a default image when we
-                    ;; added the tag.
-                    (put-text-property marker (+ marker region-length)
-                                       'display image))
-                  ;; We are done with the marker; release it:
-                  (set-marker marker nil)))
-              (kill-buffer url-buffer)))))))
+      (let* ((data (unless is-error-response-p
+                     (goto-char (point-min))
+                     (search-forward "\n\n")
+                     (buffer-substring (point) (point-max))))
+             (image (when data
+                      (apply #'create-image data
+                             (if (version< emacs-version "27.1")
+                                 (when image-options 'imagemagick)
+                               nil)     ; inbuilt scaling in 27.1
+                             t image-options))))
+        (when mastodon-media--enable-image-caching
+          (unless (url-is-cached url)   ; cache if not already cached
+            (url-store-in-cache url-buffer)))
+        (with-current-buffer (marker-buffer marker)
+          ;; Save narrowing in our buffer
+          (let ((inhibit-read-only t))
+            (save-restriction
+              (widen)
+              (put-text-property marker
+                                 (+ marker region-length) 'media-state 'loaded)
+              (when image
+                ;; We only set the image to display if we could load
+                ;; it; we already have set a default image when we
+                ;; added the tag.
+                (put-text-property marker (+ marker region-length)
+                                   'display image))
+              ;; We are done with the marker; release it:
+              (set-marker marker nil)))
+          (kill-buffer url-buffer))))))
 
 (defun mastodon-media--load-image-from-url (url media-type start region-length)
   "Take a URL and MEDIA-TYPE and load the image asynchronously.
-
-MEDIA-TYPE is a symbol and either 'avatar or 'media-link.
+MEDIA-TYPE is a symbol and either `avatar' or `media-link'.
 START is the position where we start loading the image.
 REGION-LENGTH is the range from start to propertize."
-  ;; TODO: Cache the avatars
   (let ((image-options (when (or (image-type-available-p 'imagemagick)
                                  (image-transforms-p)) ; inbuilt scaling in 27.1
                          (cond
@@ -196,8 +192,7 @@ REGION-LENGTH is the range from start to propertize."
                            `(:max-height ,mastodon-media--preview-max-height))))))
     (let ((buffer (current-buffer))
           (marker (copy-marker start))
-	  ;; Keep url.el from spamming us with messages about connecting to hosts:
-	  (url-show-status nil))
+	  (url-show-status nil)) ; stop url.el from spamming us about connecting
       (condition-case nil
           ;; catch any errors in url-retrieve so as to not abort
           ;; whatever called us
@@ -207,11 +202,12 @@ REGION-LENGTH is the range from start to propertize."
               (with-current-buffer (url-fetch-from-cache url)
                 (set-buffer-multibyte nil)
                 (goto-char (point-min))
-                (zlib-decompress-region (goto-char (search-forward "\n\n")) (point-max))
-                (mastodon-media--process-image-response nil marker image-options region-length url))
+                (zlib-decompress-region
+                 (goto-char (search-forward "\n\n")) (point-max))
+                (mastodon-media--process-image-response
+                 nil marker image-options region-length url))
             ;; else fetch as usual and process-image-response will cache it
-            (url-retrieve url
-                          #'mastodon-media--process-image-response
+            (url-retrieve url #'mastodon-media--process-image-response
                           (list marker image-options region-length url)))
         (error (with-current-buffer buffer
                  ;; TODO: Consider adding retries
@@ -223,30 +219,28 @@ REGION-LENGTH is the range from start to propertize."
 
 (defun mastodon-media--select-next-media-line (end-pos)
   "Find coordinates of the next media to load before END-POS.
-
 Returns the list of (`start' . `end', `media-symbol') points of
 that line and string found or nil no more media links were
 found."
   (let ((next-pos (point)))
-    (while (and (setq next-pos (next-single-property-change next-pos 'media-state))
-                (or (not (eq 'needs-loading (get-text-property next-pos 'media-state)))
-                    (null (get-text-property next-pos 'media-url))
-                    (null (get-text-property next-pos 'media-type))))
+    (while
+        (and
+         (setq next-pos (next-single-property-change next-pos 'media-state))
+         (or (not (eq 'needs-loading (get-text-property next-pos 'media-state)))
+             (null (get-text-property next-pos 'media-url))
+             (null (get-text-property next-pos 'media-type))))
       ;; do nothing - the loop will proceed
       )
     (when (and next-pos (< next-pos end-pos))
       (let ((media-type (get-text-property next-pos 'media-type)))
         (cond
-         ;; Avatars are just one character in the buffer
-         ((eq media-type 'avatar)
+         ((eq media-type 'avatar) ; avatars are one character
           (list next-pos (+ next-pos 1) 'avatar))
-         ;; Media links are 5 character ("[img]")
-         ((eq media-type 'media-link)
+         ((eq media-type 'media-link) ; media links are 5 characters: [img]
           (list next-pos (+ next-pos 5) 'media-link)))))))
 
 (defun mastodon-media--valid-link-p (link)
   "Check if LINK is valid.
-
 Checks to make sure the missing string has not been returned."
   (and link
        (> (length link) 8)
@@ -259,11 +253,12 @@ Replace them with the referenced image."
   (save-excursion
     (goto-char search-start)
     (let (line-details)
-      (while (setq line-details (mastodon-media--select-next-media-line
-                                 search-end))
+      (while (setq line-details
+                   (mastodon-media--select-next-media-line search-end))
         (let* ((start (car line-details))
                (end (cadr line-details))
                (media-type (cadr (cdr line-details)))
+               (type (get-text-property start 'mastodon-media-type))
                (image-url (get-text-property start 'media-url)))
           (if (not (mastodon-media--valid-link-p image-url))
               ;; mark it at least as not needing loading any more
@@ -271,21 +266,25 @@ Replace them with the referenced image."
             ;; proceed to load this image asynchronously
             (put-text-property start end 'media-state 'loading)
             (mastodon-media--load-image-from-url
-             image-url media-type start (- end start))))))))
-            ;; (mastodon-media--moving-image-overlay start end)))))))
+             image-url media-type start (- end start))
+            (when (or (equal type "gifv")
+                      (equal type "video"))
+              (mastodon-media--moving-image-overlay start end))))))))
 
-;; (defun mastodon-media--moving-image-overlay (start end)
-;;   "Add play symbol overlay to moving image media items."
-;;   (let ((ov (make-overlay start end))
-;;         (type (get-text-property start 'mastodon-media-type)))
-;;     (when (or (equal type "gifv")
-;;               (equal type "video"))
-;;       (overlay-put
-;;        ov
-;;        'after-string
-;;        (propertize " "
-;;                    'face
-;;                    '((:height 1.5 :inherit 'font-lock-comment-face)))))))
+;; (defvar-local mastodon-media--overlays nil
+;;   "Holds a list of overlays in the buffer.")
+
+(defun mastodon-media--moving-image-overlay (start end)
+  "Add play symbol overlay to moving image media items."
+  (let ((ov (make-overlay start end)))
+    (overlay-put
+     ov
+     'after-string
+     (propertize ""
+                 'help-echo "Video"
+                 'face
+                 '((:height 3.5 :inherit font-lock-comment-face))))))
+;; (cl-pushnew ov mastodon-media--overlays)))
 
 (defun mastodon-media--get-avatar-rendering (avatar-url)
   "Return the string to be written that renders the avatar at AVATAR-URL."
@@ -307,34 +306,24 @@ Replace them with the referenced image."
                                  t image-options))
      " ")))
 
-(defun mastodon-media--get-media-link-rendering (media-url &optional full-remote-url type caption)
+(defun mastodon-media--get-media-link-rendering
+    (media-url &optional full-remote-url type caption)
   "Return the string to be written that renders the image at MEDIA-URL.
 FULL-REMOTE-URL is used for `shr-browse-image'.
 TYPE is the attachment's type field on the server.
 CAPTION is the image caption if provided."
-  (let* ((help-echo-base "RET/i: load full image (prefix: copy URL), +/-: zoom, r: rotate, o: save preview")
-        (help-echo (if caption
-                       (concat help-echo-base
-                               "\n\""
-                               caption "\"")
-                     help-echo-base)))
+  (let* ((help-echo-base
+          "RET/i: load full image (prefix: copy URL), +/-: zoom,\
+ r: rotate, o: save preview")
+         (help-echo (if caption
+                        (concat help-echo-base
+                                "\n\"" caption "\"")
+                      help-echo-base)))
     (concat
-     (propertize "[img]"
-                 'media-url media-url
-                 'media-state 'needs-loading
-                 'media-type 'media-link
-                 'mastodon-media-type type
-                 'display (create-image mastodon-media--generic-broken-image-data nil t)
-                 'mouse-face 'highlight
-                 'mastodon-tab-stop 'image ; for do-link-action-at-point
-                 'image-url full-remote-url ; for shr-browse-image
-                 'keymap mastodon-tl--shr-image-map-replacement
-                 'help-echo (if (or (string= type "image")
-                                    (string= type nil)
-                                    (string= type "unknown")) ;handle borked images
-                                help-echo
-                              (concat help-echo "\nC-RET: play " type " with mpv")))
-                 " ")))
+     (mastodon-tl--propertize-img-str-or-url
+      "[img]" media-url full-remote-url type help-echo
+      (create-image mastodon-media--generic-broken-image-data nil t))
+     " ")))
 
 (provide 'mastodon-media)
 ;;; mastodon-media.el ends here
