@@ -4,6 +4,25 @@
 (require 'cl-macs)
 (require 'el-mock)
 
+(defconst mastodon-tl--test-instance-rules
+  ;; brief ones calqued off todon.nl
+  '(((id . "1")
+     (text . "We do not accept racism."))
+    ((id . "2")
+     (text . "We do not accept homophobia."))
+    ((id . "3")
+     (text . "We do not accept sexism."))
+    ((id . "4")
+     (text . "We do not accept ableism."))
+    ((id . "5")
+     (text . "We do not accept harassment."))
+    ((id . "6")
+     (text . "We also do not accept hate speech."))
+    ((id . "7")
+     (text . "We do not accept abuse of minors."))
+    ((id . "8")
+     (text . "We do not accept glorification of violence."))))
+
 (defconst mastodon-tl-test-base-toot
   '((id . 61208)
     (created_at . "2017-04-24T19:01:02.000Z")
@@ -149,16 +168,16 @@ Strict-Transport-Security: max-age=31536000
   (let ((input "foobar</p>"))
     (should (string= (mastodon-tl--remove-html input) "foobar\n\n"))))
 
-(ert-deftest mastodon-tl--toot-id-boosted ()
+(ert-deftest mastodon-tl--item-id-boosted ()
   "If a toot is boostedm, return the reblog id."
   (should (string= (mastodon-tl--as-string
-                    (mastodon-tl--toot-id mastodon-tl-test-base-boosted-toot))
+                    (mastodon-tl--item-id mastodon-tl-test-base-boosted-toot))
                    "4543919")))
 
-(ert-deftest mastodon-tl--toot-id ()
+(ert-deftest mastodon-tl--item-id ()
   "If a toot is boostedm, return the reblog id."
   (should (string= (mastodon-tl--as-string
-                    (mastodon-tl--toot-id mastodon-tl-test-base-toot))
+                    (mastodon-tl--item-id mastodon-tl-test-base-toot))
                    "61208")))
 
 (ert-deftest mastodon-tl--as-string-1 ()
@@ -175,27 +194,30 @@ Strict-Transport-Security: max-age=31536000
   "Should request toots older than max_id."
   (let ((mastodon-instance-url "https://instance.url"))
     (with-mock
-      (mock (mastodon-http--get-json "https://instance.url/api/v1/timelines/foo?max_id=12345"))
-      (mastodon-tl--more-json "timelines/foo" 12345))))
+      (mock (mastodon-http--get-json "https://instance.url/api/v1/timelines/foo"
+                                     '(("max_id" . "12345"))))
+      (mastodon-tl--more-json "timelines/foo" "12345"))))
 
 (ert-deftest mastodon-tl--more-json-id-string ()
   "Should request toots older than max_id.
 
-`mastodon-tl--more-json' should accept and id that is either
-a string or a numeric."
+  `mastodon-tl--more-json' should accept and id that is either
+  a string or a numeric."
   (let ((mastodon-instance-url "https://instance.url"))
     (with-mock
-      (mock (mastodon-http--get-json "https://instance.url/api/v1/timelines/foo?max_id=12345"))
+      (mock (mastodon-http--get-json "https://instance.url/api/v1/timelines/foo"
+                                     '(("max_id" . "12345"))))
       (mastodon-tl--more-json "timelines/foo" "12345"))))
 
 (ert-deftest mastodon-tl--update-json-id-string ()
   "Should request toots more recent than since_id.
 
-`mastodon-tl--updated-json' should accept and id that is either
-a string or a numeric."
+  `mastodon-tl--updated-json' should accept and id that is either
+  a string or a numeric."
   (let ((mastodon-instance-url "https://instance.url"))
     (with-mock
-      (mock (mastodon-http--get-json "https://instance.url/api/v1/timelines/foo?since_id=12345"))
+      (mock (mastodon-http--get-json "https://instance.url/api/v1/timelines/foo"
+                                     '(("since_id" . "12345"))))
       (mastodon-tl--updated-json "timelines/foo" "12345"))))
 
 (ert-deftest mastodon-tl--relative-time-description ()
@@ -206,32 +228,32 @@ a string or a numeric."
               (weeks (n) (* n (days 7)))
               (years (n) (* n (days 365)))
               (format-seconds-since (seconds)
-                (let ((timestamp (time-subtract (current-time) (seconds-to-time seconds))))
-                  (mastodon-tl--relative-time-description timestamp)))
+                                    (let ((timestamp (time-subtract (current-time) (seconds-to-time seconds))))
+                                      (mastodon-tl--relative-time-description timestamp)))
               (check (seconds expected)
-                (should (string= (format-seconds-since seconds) expected))))
-    (check 1 "less than a minute ago")
-    (check 59 "less than a minute ago")
-    (check 60 "one minute ago")
-    (check 89 "one minute ago")            ;; rounding down
+                     (should (string= (format-seconds-since seconds) expected))))
+    (check 1 "just now")
+    (check 59 "just now")
+    (check 60 "1 minute ago")
+    (check 89 "1 minute ago")            ;; rounding down
     (check 91 "2 minutes ago")             ;; rounding up
     (check (minutes 3.49) "3 minutes ago") ;; rounding down
     (check (minutes 3.52) "4 minutes ago")
     (check (minutes 59) "59 minutes ago")
-    (check (minutes 60) "one hour ago")
-    (check (minutes 89) "one hour ago")
+    (check (minutes 60) "1 hour ago")
+    (check (minutes 89) "1 hour ago")
     (check (minutes 91) "2 hours ago")
     (check (hours 3.49) "3 hours ago") ;; rounding down
     (check (hours 3.51) "4 hours ago") ;; rounding down
     (check (hours 23.4) "23 hours ago")
-    (check (hours 23.6) "one day ago") ;; rounding up
-    (check (days 1.48) "one day ago")  ;; rounding down
+    (check (hours 23.6) "1 day ago") ;; rounding up
+    (check (days 1.48) "1 day ago")  ;; rounding down
     (check (days 1.52) "2 days ago")   ;; rounding up
-    (check (days 6.6) "one week ago")  ;; rounding up
+    (check (days 6.6) "1 week ago")  ;; rounding up
     (check (weeks 2.49) "2 weeks ago") ;; rounding down
     (check (weeks 2.51) "3 weeks ago") ;; rounding down
     (check (1- (weeks 52)) "52 weeks ago")
-    (check (weeks 52) "one year ago")
+    (check (weeks 52) "1 year ago")
     (check (years 2.49) "2 years ago") ;; rounding down
     (check (years 2.51) "3 years ago") ;; rounding down
     ))
@@ -245,33 +267,33 @@ a string or a numeric."
                 (weeks (n) (* n (days 7)))
                 (years (n) (* n (days 365.25)))
                 (next-update (seconds-ago)
-                  (let* ((timestamp (time-subtract current-time
-                                                   (seconds-to-time seconds-ago))))
-                    (cdr (mastodon-tl--relative-time-details timestamp current-time))))
+                             (let* ((timestamp (time-subtract current-time
+                                                              (seconds-to-time seconds-ago))))
+                               (cdr (mastodon-tl--relative-time-details timestamp current-time))))
                 (check (seconds-ago)
-                  (let* ((timestamp (time-subtract current-time (seconds-to-time seconds-ago)))
-                         (at-now (mastodon-tl--relative-time-description timestamp current-time))
-                         (at-one-second-before (mastodon-tl--relative-time-description
-                                                timestamp
-                                                (time-subtract (next-update seconds-ago)
-                                                               (seconds-to-time 1))))
-                         (at-result (mastodon-tl--relative-time-description
-                                     timestamp
-                                     (next-update seconds-ago))))
-                    (when nil  ;; change to t to debug test failures
-                      (prin1 (format "\nFor %s: %s / %s"
-                                     seconds-ago
-                                     (time-to-seconds
-                                      (time-subtract (next-update seconds-ago)
-                                                     timestamp))
-                                     (round
-                                      (time-to-seconds
-                                       (time-subtract (next-update seconds-ago)
-                                                      current-time))))))
-                    ;; a second earlier the description is the same as at current time
-                    (should (string= at-now at-one-second-before))
-                    ;; but at the result time it is different
-                    (should-not (string= at-one-second-before at-result)))))
+                       (let* ((timestamp (time-subtract current-time (seconds-to-time seconds-ago)))
+                              (at-now (mastodon-tl--relative-time-description timestamp current-time))
+                              (at-one-second-before (mastodon-tl--relative-time-description
+                                                     timestamp
+                                                     (time-subtract (next-update seconds-ago)
+                                                                    (seconds-to-time 1))))
+                              (at-result (mastodon-tl--relative-time-description
+                                          timestamp
+                                          (next-update seconds-ago))))
+                         (when nil  ;; change to t to debug test failures
+                           (prin1 (format "\nFor %s: %s / %s"
+                                          seconds-ago
+                                          (time-to-seconds
+                                           (time-subtract (next-update seconds-ago)
+                                                          timestamp))
+                                          (round
+                                           (time-to-seconds
+                                            (time-subtract (next-update seconds-ago)
+                                                           current-time))))))
+                         ;; a second earlier the description is the same as at current time
+                         (should (string= at-now at-one-second-before))
+                         ;; but at the result time it is different
+                         (should-not (string= at-one-second-before at-result)))))
       (check 0)
       (check 1)
       (check 59)
@@ -304,6 +326,7 @@ a string or a numeric."
         (timestamp (cdr (assoc 'created_at mastodon-tl-test-base-toot))))
     (with-mock
       (mock (date-to-time timestamp) => '(22782 21551))
+      (mock (mastodon-tl--toot-stats mastodon-tl-test-base-toot) => "")
       (mock (format-time-string mastodon-toot-timestamp-format '(22782 21551)) => "2999-99-99 00:11:22")
 
       (let ((byline (mastodon-tl--byline mastodon-tl-test-base-toot
@@ -312,9 +335,10 @@ a string or a numeric."
 	    (handle-location 20))
         (should (string= (substring-no-properties
 			  byline)
-			 "Account 42 (@acct42@example.space) 2999-99-99 00:11:22
-  ------------
-"))
+			 (concat "Account 42 (@acct42@example.space) 2999-99-99 00:11:22
+  "
+                                 mastodon-tl--horiz-bar "
+")))
         (should (eq (get-text-property handle-location 'mastodon-tab-stop byline)
                     'user-handle))
         (should (string= (get-text-property handle-location 'mastodon-handle byline)
@@ -329,15 +353,17 @@ a string or a numeric."
     (with-mock
       (stub create-image => '(image "fake data"))
       (mock (date-to-time timestamp) => '(22782 21551))
+      (mock (mastodon-tl--toot-stats mastodon-tl-test-base-toot) => "")
       (mock (format-time-string mastodon-toot-timestamp-format '(22782 21551)) => "2999-99-99 00:11:22")
 
       (should (string= (substring-no-properties
                         (mastodon-tl--byline mastodon-tl-test-base-toot
                                              'mastodon-tl--byline-author
                                              'mastodon-tl--byline-boosted))
-                       "Account 42 (@acct42@example.space) 2999-99-99 00:11:22
-  ------------
-")))))
+                       (concat "Account 42 (@acct42@example.space) 2999-99-99 00:11:22
+  "
+                               mastodon-tl--horiz-bar "
+"))))))
 
 (ert-deftest mastodon-tl--byline-boosted ()
   "Should format the boosted toot correctly."
@@ -346,15 +372,18 @@ a string or a numeric."
          (timestamp (cdr (assoc 'created_at toot))))
     (with-mock
       (mock (date-to-time timestamp) => '(22782 21551))
+      (mock (mastodon-tl--symbol 'boost) => "B")
+      (mock (mastodon-tl--toot-stats toot) => "")
       (mock (format-time-string mastodon-toot-timestamp-format '(22782 21551)) => "2999-99-99 00:11:22")
 
       (should (string= (substring-no-properties
                         (mastodon-tl--byline toot
                                              'mastodon-tl--byline-author
                                              'mastodon-tl--byline-boosted))
-                       "(B) Account 42 (@acct42@example.space) 2999-99-99 00:11:22
-  ------------
-")))))
+                       (concat "(B) Account 42 (@acct42@example.space) 2999-99-99 00:11:22
+  "
+                               mastodon-tl--horiz-bar "
+"))))))
 
 (ert-deftest mastodon-tl--byline-favorited ()
   "Should format the favourited toot correctly."
@@ -362,16 +391,19 @@ a string or a numeric."
          (toot (cons '(favourited . t) mastodon-tl-test-base-toot))
          (timestamp (cdr (assoc 'created_at toot))))
     (with-mock
+      (mock (mastodon-tl--symbol 'favourite) => "F")
       (mock (date-to-time timestamp) => '(22782 21551))
+      (mock (mastodon-tl--toot-stats toot) => "")
       (mock (format-time-string mastodon-toot-timestamp-format '(22782 21551)) => "2999-99-99 00:11:22")
 
       (should (string= (substring-no-properties
                         (mastodon-tl--byline toot
                                              'mastodon-tl--byline-author
                                              'mastodon-tl--byline-boosted))
-                       "(F) Account 42 (@acct42@example.space) 2999-99-99 00:11:22
-  ------------
-")))))
+                       (concat "(F) Account 42 (@acct42@example.space) 2999-99-99 00:11:22
+  "
+                               mastodon-tl--horiz-bar "
+"))))))
 
 
 (ert-deftest mastodon-tl--byline-boosted/favorited ()
@@ -380,16 +412,22 @@ a string or a numeric."
          (toot `((favourited . t) (reblogged . t) ,@mastodon-tl-test-base-toot))
          (timestamp (cdr (assoc 'created_at toot))))
     (with-mock
+      (mock (mastodon-tl--toot-stats toot) => "")
       (mock (date-to-time timestamp) => '(22782 21551))
+      ;; FIXME this mock refuses to recognise our different args
+      ;; (mock (mastodon-tl--symbol 'favourite) => "F")
+      ;; (mock (mastodon-tl--symbol 'boost) => "B")
+      (mock (mastodon-tl--symbol *) => "?")
       (mock (format-time-string mastodon-toot-timestamp-format '(22782 21551)) => "2999-99-99 00:11:22")
 
       (should (string= (substring-no-properties
                         (mastodon-tl--byline toot
                                              'mastodon-tl--byline-author
                                              'mastodon-tl--byline-boosted))
-                       "(B) (F) Account 42 (@acct42@example.space) 2999-99-99 00:11:22
-  ------------
-")))))
+                       (concat "(?) (?) Account 42 (@acct42@example.space) 2999-99-99 00:11:22
+  "
+                               mastodon-tl--horiz-bar "
+"))))))
 
 (ert-deftest mastodon-tl--byline-reblogged ()
   "Should format the reblogged toot correctly."
@@ -404,6 +442,7 @@ a string or a numeric."
       (mock (date-to-time timestamp) => '(1 2))
       (mock (format-time-string mastodon-toot-timestamp-format '(1 2)) => "reblogging time")
       (mock (date-to-time original-timestamp) => '(3 4))
+      (mock (mastodon-tl--toot-stats toot) => "")
       (mock (format-time-string mastodon-toot-timestamp-format '(3 4)) => "original time")
 
       (let ((byline (mastodon-tl--byline toot
@@ -412,10 +451,10 @@ a string or a numeric."
 	    (handle1-location 20)
 	    (handle2-location 65))
         (should (string= (substring-no-properties byline)
-			 "Account 42 (@acct42@example.space)
- Boosted Account 43 (@acct43@example.space) original time
-  ------------
-"))
+			 (concat "Account 42 (@acct42@example.space)
+  Boosted Account 43 (@acct43@example.space) original time
+  " mastodon-tl--horiz-bar "
+")))
         (should (eq (get-text-property handle1-location 'mastodon-tab-stop byline)
                     'user-handle))
         (should (equal (get-text-property handle1-location 'help-echo byline)
@@ -440,15 +479,16 @@ a string or a numeric."
       (mock (format-time-string mastodon-toot-timestamp-format '(1 2)) => "reblogging time")
       (mock (date-to-time original-timestamp) => '(3 4))
       (mock (format-time-string mastodon-toot-timestamp-format '(3 4)) => "original time")
+      (mock (mastodon-tl--toot-stats toot) => "")
 
       (should (string= (substring-no-properties
                         (mastodon-tl--byline toot
                                              'mastodon-tl--byline-author
                                              'mastodon-tl--byline-boosted))
-                       "Account 42 (@acct42@example.space)
- Boosted Account 43 (@acct43@example.space) original time
-  ------------
-")))))
+                       (concat "Account 42 (@acct42@example.space)
+  Boosted Account 43 (@acct43@example.space) original time
+  " mastodon-tl--horiz-bar "
+"))))))
 
 (ert-deftest mastodon-tl--byline-reblogged-boosted/favorited ()
   "Should format the reblogged toot that was also boosted & favoritedcorrectly."
@@ -461,6 +501,11 @@ a string or a numeric."
       ;; We don't expect to use the toot's timestamp but the timestamp of the
       ;; reblogged toot:
       (mock (date-to-time timestamp) => '(1 2))
+      ;; FIXME this mock refuses to recognise our different args
+      ;; (mock (mastodon-tl--symbol 'favourite) => "F")
+      ;; (mock (mastodon-tl--symbol 'boost) => "B")
+      (mock (mastodon-tl--symbol *) => "?")
+      (mock (mastodon-tl--toot-stats toot) => "")
       (mock (format-time-string mastodon-toot-timestamp-format '(1 2)) => "reblogging time")
       (mock (date-to-time original-timestamp) => '(3 4))
       (mock (format-time-string mastodon-toot-timestamp-format '(3 4)) => "original time")
@@ -469,10 +514,10 @@ a string or a numeric."
                         (mastodon-tl--byline toot
                                              'mastodon-tl--byline-author
                                              'mastodon-tl--byline-boosted))
-                       "(B) (F) Account 42 (@acct42@example.space)
- Boosted Account 43 (@acct43@example.space) original time
-  ------------
-")))))
+                       (concat "(?) (?) Account 42 (@acct42@example.space)
+  Boosted Account 43 (@acct43@example.space) original time
+  " mastodon-tl--horiz-bar "
+"))))))
 
 (ert-deftest mastodon-tl--byline-timestamp-has-relative-display ()
   "Should display the timestamp with a relative time."
@@ -804,10 +849,8 @@ a string or a numeric."
               (should (equal (list 'next test-name expected-next)
                              (list 'next test-name (point)))))))))))
 
-
 (defun tl-tests--property-values-at (property ranges)
   "Returns a list with property values at the given ranges.
-
 The property value for PROPERTY within a region is assumed to be
 constant."
   (let (result)
@@ -819,13 +862,13 @@ constant."
   (let ((now (current-time))
         markers)
     (cl-labels ((insert-timestamp (n)
-                  (insert (format "\nSome text before timestamp %s:" n))
-                  (insert (propertize
-                           (format "timestamp #%s" n)
-                           'timestamp (time-subtract now (seconds-to-time (* 60 n)))
-                           'display (format "unset %s" n)))
-                  (push (copy-marker (point)) markers)
-                  (insert " some more text.")))
+                                  (insert (format "\nSome text before timestamp %s:" n))
+                                  (insert (propertize
+                                           (format "timestamp #%s" n)
+                                           'timestamp (time-subtract now (seconds-to-time (* 60 n)))
+                                           'display (format "unset %s" n)))
+                                  (push (copy-marker (point)) markers)
+                                  (insert " some more text.")))
       (with-temp-buffer
         (cl-dotimes (n 12) (insert-timestamp (+ n 2)))
         (setq markers (nreverse markers))
@@ -883,16 +926,19 @@ constant."
       (insert "some text before\n")
       (setq toot-start (point))
       (with-mock
-        (stub create-image => '(image "fake data"))
-        (stub shr-render-region => nil) ;; Travis's Emacs doesn't have libxml
-        (insert
-         (mastodon-tl--spoiler normal-toot-with-spoiler)))
+       (mock (mastodon-profile--get-preferences-pref
+              'reading:expand:spoilers)
+             => :json-false)
+       (stub create-image => '(image "fake data"))
+       (stub shr-render-region => nil) ;; Travis's Emacs doesn't have libxml
+       (insert
+        (mastodon-tl--spoiler normal-toot-with-spoiler)))
       (setq toot-end (point))
       (insert "\nsome more text.")
       (add-text-properties
        toot-start toot-end
-       (list 'toot-json normal-toot-with-spoiler
-	     'toot-id (cdr (assoc 'id normal-toot-with-spoiler))))
+       (list 'item-json normal-toot-with-spoiler
+	     'item-id (cdr (assoc 'id normal-toot-with-spoiler))))
 
       (goto-char toot-start)
       ;; (should (eq t (looking-at "This is the spoiler warning text")))
@@ -964,28 +1010,28 @@ constant."
 (ert-deftest mastodon-tl--extract-hashtag-from-url-mastodon-link ()
   "Should extract the hashtag from a tags url."
   (should (equal (mastodon-tl--extract-hashtag-from-url
-		  "https://example.org/tags/foo"
-		  "https://example.org")
-		 "foo")))
+		          "https://example.org/tags/foo"
+		          "https://example.org")
+		         "foo")))
 
 (ert-deftest mastodon-tl--extract-hashtag-from-url-other-link ()
   "Should extract the hashtag from a tag url."
   (should (equal (mastodon-tl--extract-hashtag-from-url
-		  "https://example.org/tag/foo"
-		  "https://example.org")
-		 "foo")))
+		          "https://example.org/tag/foo"
+		          "https://example.org")
+		         "foo")))
 
 (ert-deftest mastodon-tl--extract-hashtag-from-url-wrong-instance ()
   "Should not find a tag when the instance doesn't match."
   (should (null (mastodon-tl--extract-hashtag-from-url
-		 "https://example.org/tags/foo"
-		 "https://other.example.org"))))
+		         "https://example.org/tags/foo"
+		         "https://other.example.org"))))
 
 (ert-deftest mastodon-tl--extract-hashtag-from-url-not-tag ()
   "Should not find a hashtag when not a tag url"
   (should (null (mastodon-tl--extract-hashtag-from-url
-		 "https://example.org/@userid"
-		 "https://example.org"))))
+		         "https://example.org/@userid"
+		         "https://example.org"))))
 
 (ert-deftest mastodon-tl--userhandles ()
   "Should recognise userhandles in a toot and add the required properties to it."
@@ -1044,7 +1090,7 @@ correct value for following, as well as notifications enabled or disabled."
       (let ((response-buffer-true (current-buffer)))
         (insert mastodon-tl--follow-notify-true-response)
         (with-mock
-          (mock (mastodon-http--post url-follow-only nil nil)
+          (mock (mastodon-http--post url-follow-only nil)
                 => response-buffer-true)
           (should
            (equal
@@ -1052,8 +1098,12 @@ correct value for following, as well as notifications enabled or disabled."
                                                   user-name
                                                   user-handle
                                                   "follow")
-            "User some-user (@some-user@instance.url) followed!"))
-          (mock (mastodon-http--post url-mute nil nil)
+            "User some-user (@some-user@instance.url) followed!")))))
+    (with-temp-buffer
+      (let ((response-buffer-true (current-buffer)))
+        (insert mastodon-tl--follow-notify-true-response)
+        (with-mock
+          (mock (mastodon-http--post url-mute nil)
                 => response-buffer-true)
           (should
            (equal
@@ -1061,8 +1111,12 @@ correct value for following, as well as notifications enabled or disabled."
                                                   user-name
                                                   user-handle
                                                   "mute")
-            "User some-user (@some-user@instance.url) muted!"))
-          (mock (mastodon-http--post url-block nil nil)
+            "User some-user (@some-user@instance.url) muted!")))))
+    (with-temp-buffer
+      (let ((response-buffer-true (current-buffer)))
+        (insert mastodon-tl--follow-notify-true-response)
+        (with-mock
+          (mock (mastodon-http--post url-block nil)
                 => response-buffer-true)
           (should
            (equal
@@ -1070,22 +1124,26 @@ correct value for following, as well as notifications enabled or disabled."
                                                   user-name
                                                   user-handle
                                                   "block")
-            "User some-user (@some-user@instance.url) blocked!")))
+            "User some-user (@some-user@instance.url) blocked!")))))
+    (with-temp-buffer
+      (let ((response-buffer-true (current-buffer)))
+        (insert mastodon-tl--follow-notify-true-response)
         (with-mock
-          (mock (mastodon-http--post url-true nil nil) => response-buffer-true)
-          (should
-           (equal
-            (mastodon-tl--do-user-action-function url-true
-                                                  user-name
-                                                  user-handle
-                                                  "follow"
-                                                  "true")
-            "Receiving notifications for user some-user (@some-user@instance.url)!")))))
+          (with-mock
+            (mock (mastodon-http--post url-true nil) => response-buffer-true)
+            (should
+             (equal
+              (mastodon-tl--do-user-action-function url-true
+                                                    user-name
+                                                    user-handle
+                                                    "follow"
+                                                    "true")
+              "Receiving notifications for user some-user (@some-user@instance.url)!"))))))
     (with-temp-buffer
       (let ((response-buffer-false (current-buffer)))
         (insert mastodon-tl--follow-notify-false-response)
         (with-mock
-          (mock (mastodon-http--post url-false nil nil) => response-buffer-false)
+          (mock (mastodon-http--post url-false nil) => response-buffer-false)
           (should
            (equal
             (mastodon-tl--do-user-action-function url-false
@@ -1094,3 +1152,85 @@ correct value for following, as well as notifications enabled or disabled."
                                                   "follow"
                                                   "false")
             "Not receiving notifications for user some-user (@some-user@instance.url)!")))))))
+
+(ert-deftest mastodon-tl--report-to-mods-params-alist ()
+  ""
+  (with-temp-buffer
+    (let* ((toot mastodon-tl-test-base-toot)
+           (account (alist-get 'account toot)))
+      (with-mock
+       ;; no longer needed after our refactor
+       ;; (mock (mastodon-http--api "reports") => "https://instance.url/api/v1/reports")
+       ;; (mock (mastodon-tl--toot-or-base
+       ;; (mastodon-tl--property 'item-json :no-move))
+       ;; => mastodon-tl-test-base-toot)
+       (mock (read-string "Add comment [optional]: ") => "Dummy complaint")
+       (stub y-or-n-p => nil) ; no to all
+       (should (equal (mastodon-tl--report-params account toot)
+                      '(("account_id" . 42)
+                        ("comment" . "Dummy complaint")
+                        ("category" . "other"))))
+       (with-mock
+        (stub y-or-n-p => t) ; yes to all
+        (mock (mastodon-tl--read-rules-ids) => '(1 2 3))
+        (should (equal (mastodon-tl--report-params account toot)
+                       '(("rule_ids[]" . 3)
+                         ("rule_ids[]" . 2)
+                         ("rule_ids[]" . 1)
+                         ("account_id" . 42)
+                         ("comment" . "Dummy complaint")
+                         ("status_ids[]" . 61208)
+                         ("forward" . "true")))))))))
+
+(ert-deftest mastodon-tl--report-build-params ()
+  ""
+  (should (equal
+           (mastodon-tl--report-build-params 42 "Dummy complaint"
+                                             61208 "true" nil '(1 2 3))
+           '(("rule_ids[]" . 3)
+             ("rule_ids[]" . 2)
+             ("rule_ids[]" . 1)
+             ("account_id" . 42)
+             ("comment" . "Dummy complaint")
+             ("status_ids[]" . 61208)
+             ("forward" . "true"))))
+  (should (equal
+           (mastodon-tl--report-build-params 42 "Dummy complaint"
+                                             nil "true" nil nil)
+           '(("account_id" . 42)
+             ("comment" . "Dummy complaint")
+             ("forward" . "true"))))
+  (should (equal
+           (mastodon-tl--report-build-params 42 "Dummy complaint"
+                                             61208 "true" "spam" nil)
+           '(("account_id" . 42)
+             ("comment" . "Dummy complaint")
+             ("status_ids[]" . 61208)
+             ("forward" . "true")
+             ("category" . "spam"))))
+  (should (equal
+           (mastodon-tl--report-build-params 42 "Dummy complaint"
+                                             61208 "true" "other" nil)
+           '(("account_id" . 42)
+             ("comment" . "Dummy complaint")
+             ("status_ids[]" . 61208)
+             ("forward" . "true")
+             ("category" . "other"))))
+  (should (equal
+           (mastodon-tl--report-build-params 42 "Dummy complaint"
+                                             61208 nil "spam" nil)
+           '(("account_id" . 42)
+             ("comment" . "Dummy complaint")
+             ("status_ids[]" . 61208)
+             ("category" . "spam")))))
+
+(ert-deftest mastodon-tl--read-rules ()
+  "Should return a list of string numbers based on `mastodon-tl--test-instance-rules'"
+  (let ((crm-separator "[ 	]*,[ 	]*"))
+    (with-mock
+      (stub mastodon-tl--instance-rules => mastodon-tl--test-instance-rules)
+      (stub completing-read-multiple => '("We do not accept homophobia."
+                                          "We do not accept harassment."
+                                          "We also do not accept hate speech."))
+      (should (equal '("2" "5" "6")
+                     (mastodon-tl--read-rules-ids))))))
